@@ -3,14 +3,14 @@
 # pyglet
 # Copyright (c) 2006-2008 Alex Holkner
 # All rights reserved.
-# 
+#
 # Redistribution and use in source and binary forms, with or without
-# modification, are permitted provided that the following conditions 
+# modification, are permitted provided that the following conditions
 # are met:
 #
 #  * Redistributions of source code must retain the above copyright
 #    notice, this list of conditions and the following disclaimer.
-#  * Redistributions in binary form must reproduce the above copyright 
+#  * Redistributions in binary form must reproduce the above copyright
 #    notice, this list of conditions and the following disclaimer in
 #    the documentation and/or other materials provided with the
 #    distribution.
@@ -36,8 +36,10 @@
 '''
 '''
 
+from __future__ import print_function
+
 __docformat__ = 'restructuredtext'
-__version__ = '$Id: events.py 1966 2008-03-23 07:01:26Z Alex.Holkner $'
+__version__ = '$Id$'
 
 import pyglet
 from pyglet.window import key
@@ -48,16 +50,16 @@ window = pyglet.window.Window()
 @window.event
 def on_key_press(symbol, modifiers):
     if symbol == key.A:
-        print 'The "A" key was pressed.'
+        print('The "A" key was pressed.')
     elif symbol == key.LEFT:
-        print 'The left arrow key was pressed.'
+        print('The left arrow key was pressed.')
     elif symbol == key.ENTER:
-        print 'The enter key was pressed.'
+        print('The enter key was pressed.')
 
 @window.event
 def on_mouse_press(x, y, button, modifiers):
     if button == mouse.LEFT:
-        print 'The left mouse button was pressed.'
+        print('The left mouse button was pressed.')
 
 @window.event
 def on_draw():
