@@ -3,14 +3,14 @@
 # pyglet
 # Copyright (c) 2006-2008 Alex Holkner
 # All rights reserved.
-# 
+#
 # Redistribution and use in source and binary forms, with or without
-# modification, are permitted provided that the following conditions 
+# modification, are permitted provided that the following conditions
 # are met:
 #
 #  * Redistributions of source code must retain the above copyright
 #    notice, this list of conditions and the following disclaimer.
-#  * Redistributions in binary form must reproduce the above copyright 
+#  * Redistributions in binary form must reproduce the above copyright
 #    notice, this list of conditions and the following disclaimer in
 #    the documentation and/or other materials provided with the
 #    distribution.
@@ -32,26 +32,23 @@
 # ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 # POSSIBILITY OF SUCH DAMAGE.
 # ----------------------------------------------------------------------------
-# $Id: soundspace.py 2529 2009-10-12 17:39:44Z benjamin.coder.smith@gmail.com $
+# $Id$
 
 import math
 import os
 
+import pyglet
 from pyglet.gl import *
-from pyglet import font
-from pyglet import media
-from pyglet import resource
-from pyglet import window
 
 import reader
 
-resource.path.append('res')
-resource.reindex()
+pyglet.resource.path.append('res')
+pyglet.resource.reindex()
 
 # Check for AVbin
-try:
-    from pyglet.media import avbin
-except ImportError:
+from pyglet.media import have_avbin
+
+if not have_avbin():
     raise ImportError('AVbin is required for this example, see '
         'http://code.google.com/p/avbin')
 
@@ -110,14 +107,14 @@ class Handle(object):
 class LabelHandle(Handle):
     def __init__(self, player):
         super(LabelHandle, self).__init__(player)
-        self.text = font.Text(font.load('', 10), '', color=(0, 0, 0, 1),
-                              valign='top', halign='center')
+        self.text = pyglet.text.Label('', font_size=10, color=(0, 0, 0, 255),
+                                      anchor_y='top', anchor_x='center')
 
     def hit_test(self, x, y, z):
         return None
 
     def draw(self):
-        if hasattr(player, 'label'):
+        if hasattr(self.player, 'label'):
             x, _, y = self.player.position
 
             # ech. fudge scale back to 1
@@ -155,8 +152,8 @@ class PositionHandle(Handle):
     def on_mouse_drag(self, x, y, dx, dy, buttons, modifiers):
         pos = self.win.mouse_transform(x, y)
         self.player.position = \
-            (pos[0] - self.offset[0], 
-             pos[1] - self.offset[1], 
+            (pos[0] - self.offset[0],
+             pos[1] - self.offset[1],
              pos[2] - self.offset[2])
 
 class OrientationHandle(Handle):
@@ -185,8 +182,8 @@ class OrientationHandle(Handle):
         glBegin(GL_LINES)
         glVertex2f(px, py)
         glVertex2f(x, y)
-        glEnd()        
-        
+        glEnd()
+
         # This handle (orientation)
         glColor3f(1, 1, 0)
         disc(self.radius, x, y)
@@ -229,7 +226,7 @@ class ConeAngleHandle(Handle):
         x = math.cos(angle) * self.length
         z = math.sin(angle) * self.length
         return px + x, py, pz + z
-       
+
     def draw(self):
         glPushAttrib(GL_ENABLE_BIT | GL_CURRENT_BIT)
 
@@ -240,7 +237,7 @@ class ConeAngleHandle(Handle):
         px, _, py = self.player.position
         angle = orientation_angle(self.player.cone_orientation)
         a = self.get_angle() * math.pi / 180.
-        disc(self.length, px, py, 
+        disc(self.length, px, py,
              start=angle - a/2,
              end=angle + a/2)
 
@@ -349,7 +346,7 @@ class MoreHandle(Handle):
         x, y, z = self.win.mouse_transform(x, y)
         for handle in self.win.more_handles:
             if handle.hit_test(x, y, z):
-                return 
+                return
         self.win.set_more_player_handles(None)
         self.win.remove_handlers(self)
         self.open = False
@@ -458,7 +455,7 @@ class ConeOuterGainHandle(SliderHandle):
     def set_value(self, value):
         self.player.cone_outer_gain = value
 
-class SoundSpaceWindow(window.Window):
+class SoundSpaceWindow(pyglet.window.Window):
     def __init__(self, **kwargs):
         kwargs.update(dict(
             caption='Sound Space',
@@ -470,13 +467,14 @@ class SoundSpaceWindow(window.Window):
         self.handles = []
         self.more_handles = []
 
-        self.handles.append(PositionHandle(media.listener))
-        self.handles.append(ForwardOrientationHandle(media.listener))
-        self.handles.append(ListenerVolumeHandle(media.listener))
-        self.handles.append(LabelHandle(media.listener))
+        listener = pyglet.media.get_audio_driver().get_listener()
+        self.handles.append(PositionHandle(listener))
+        self.handles.append(ForwardOrientationHandle(listener))
+        self.handles.append(ListenerVolumeHandle(listener))
+        self.handles.append(LabelHandle(listener))
 
-        self.tip = font.Text(font.load('', 10), '', color=(0, 0, 0, 1),
-                             halign='center', valign='top')
+        self.tip = pyglet.text.Label('', font_size=10, color=(0, 0, 0, 255),
+                                     anchor_y='top', anchor_x='center')
         self.tip_player = None
 
         # pixels per unit
@@ -524,7 +522,7 @@ class SoundSpaceWindow(window.Window):
         glScalef(self.zoom, self.zoom, 1)
 
     def mouse_transform(self, x, y):
-        return (float(x - self.tx) / self.zoom, 
+        return (float(x - self.tx) / self.zoom,
                 0,
                 float(y - self.ty) / self.zoom)
 
@@ -540,27 +538,22 @@ class SoundSpaceWindow(window.Window):
                 return handle, offset
         return None, None
 
-    def run(self):
-        while not self.has_exit:
-            for player in self.players:
-                player.dispatch_events()
-            self.dispatch_events()
-            glClearColor(.8, .8, .8, 1)
-            self.clear()
-            self.draw_background()
+    def on_draw(self):
+        glClearColor(.8, .8, .8, 1)
+        self.clear()
+        self.draw_background()
 
-            glPushMatrix()
-            self.camera_transform()
-            for handle in self.handles + self.more_handles:
-                handle.draw()
-            glPopMatrix()
+        glPushMatrix()
+        self.camera_transform()
+        for handle in self.handles + self.more_handles:
+            handle.draw()
+        glPopMatrix()
 
-            if self.tip_player:
-                player_pos = self.player_transform(self.tip_player)
-                self.tip.x = player_pos[0]
-                self.tip.y = player_pos[1] - 15
-                self.tip.draw()
-            self.flip()
+        if self.tip_player:
+            player_pos = self.player_transform(self.tip_player)
+            self.tip.x = player_pos[0]
+            self.tip.y = player_pos[1] - 15
+            self.tip.draw()
 
     def on_mouse_scroll(self, x, y, dx, dy):
         self.zoom += dy * 10
@@ -598,17 +591,18 @@ class PanView(object):
 
 if __name__ == '__main__':
     # We swap Y and Z, moving to left-handed system
-    media.listener.up_orientation = (0, -1, 0)
-    
-    # Start facing up (er, forwards)
-    media.listener.forward_orientation = (0, 0, 1)
+    listener = pyglet.media.get_audio_driver().get_listener()
+    listener.up_orientation = (0, -1, 0)
 
-    media.listener.label = 'Listener'
+    # Start facing up (er, forwards)
+    listener.forward_orientation = (0, 0, 1)
+
+    listener.label = 'Listener'
 
     w = SoundSpaceWindow()
     r = reader.SpaceReader(w)
-    r.read(resource.file('space.txt'))
-    for player in w.players:
-        player.play()
-        pass
-    w.run()
+    r.read(pyglet.resource.file('space.txt'))
+    player_group = pyglet.media.PlayerGroup(w.players)
+    player_group.play()
+
+    pyglet.app.run()
