@@ -2,14 +2,14 @@
 # pyglet
 # Copyright (c) 2006-2008 Alex Holkner
 # All rights reserved.
-# 
+#
 # Redistribution and use in source and binary forms, with or without
-# modification, are permitted provided that the following conditions 
+# modification, are permitted provided that the following conditions
 # are met:
 #
 #  * Redistributions of source code must retain the above copyright
 #    notice, this list of conditions and the following disclaimer.
-#  * Redistributions in binary form must reproduce the above copyright 
+#  * Redistributions in binary form must reproduce the above copyright
 #    notice, this list of conditions and the following disclaimer in
 #    the documentation and/or other materials provided with the
 #    distribution.
@@ -38,7 +38,7 @@
 A vertex "domain" consists of a set of attribute descriptions that together
 describe the layout of one or more vertex buffers which are used together to
 specify the vertices in a primitive.  Additionally, the domain manages the
-buffers used to store the data and will resize them as necessary to accomodate
+buffers used to store the data and will resize them as necessary to accommodate
 new vertices.
 
 Domains can optionally be indexed, in which case they also manage a buffer
@@ -46,15 +46,18 @@ containing vertex indices.  This buffer is grown separately and has no size
 relation to the attribute buffers.
 
 Applications can create vertices (and optionally, indices) within a domain
-with the `VertexDomain.create` method.  This returns a `VertexList`
-representing the list of vertices created.  The vertex attribute data within
-the group can be modified, and the changes will be made to the underlying
-buffers automatically.
+with the :py:meth:`VertexDomain.create` method.  This returns a
+:py:class:`VertexList` representing the list of vertices created.  The vertex
+attribute data within the group can be modified, and the changes will be made
+to the underlying buffers automatically.
 
 The entire domain can be efficiently drawn in one step with the
-`VertexDomain.draw` method, assuming all the vertices comprise primitives of
-the same OpenGL primitive mode.
+:py:meth:`VertexDomain.draw` method, assuming all the vertices comprise
+primitives of the same OpenGL primitive mode.
 '''
+from builtins import zip
+from builtins import range
+from builtins import object
 
 __docformat__ = 'restructuredtext'
 __version__ = '$Id: $'
@@ -99,7 +102,7 @@ def create_attribute_usage(format):
     to the OpenGL VBO usage hint, and for ``static`` also indicates a
     preference for interleaved arrays.  If ``none`` is specified a buffer
     object is not created, and vertex data is stored in system memory.
-    
+
     Some examples:
 
     ``v3f/stream``
@@ -107,7 +110,7 @@ def create_attribute_usage(format):
     ``c4b/static``
         4-byte color attribute, for static usage
 
-    :return: attribute, usage  
+    :return: attribute, usage
     '''
     match = _usage_format_re.match(format)
     attribute_format = match.group('attribute')
@@ -124,11 +127,11 @@ def create_attribute_usage(format):
 
 def create_domain(*attribute_usage_formats):
     '''Create a vertex domain covering the given attribute usage formats.
-    See documentation for `create_attribute_usage` and 
-    `pyglet.graphics.vertexattribute.create_attribute` for the grammar of
-    these format strings.
+    See documentation for :py:func:`create_attribute_usage` and
+    :py:func:`pyglet.graphics.vertexattribute.create_attribute` for the grammar
+    of these format strings.
 
-    :rtype: `VertexDomain`
+    :rtype: :py:class:`VertexDomain`
     '''
     attribute_usages = [create_attribute_usage(f) \
                         for f in attribute_usage_formats]
@@ -136,11 +139,11 @@ def create_domain(*attribute_usage_formats):
 
 def create_indexed_domain(*attribute_usage_formats):
     '''Create an indexed vertex domain covering the given attribute usage
-    formats.  See documentation for `create_attribute_usage` and
-    `pyglet.graphics.vertexattribute.create_attribute` for the grammar of
-    these format strings.
+    formats.  See documentation for :py:class:`create_attribute_usage` and
+    :py:func:`pyglet.graphics.vertexattribute.create_attribute` for the grammar
+    of these format strings.
 
-    :rtype: `VertexDomain`
+    :rtype: :py:class:`VertexDomain`
     '''
     attribute_usages = [create_attribute_usage(f) \
                         for f in attribute_usage_formats]
@@ -149,8 +152,8 @@ def create_indexed_domain(*attribute_usage_formats):
 class VertexDomain(object):
     '''Management of a set of vertex lists.
 
-    Construction of a vertex domain is usually done with the `create_domain`
-    function.
+    Construction of a vertex domain is usually done with the
+    :py:func:`create_domain` function.
     '''
     _version = 0
     _initial_count = 16
@@ -158,10 +161,22 @@ class VertexDomain(object):
     def __init__(self, attribute_usages):
         self.allocator = allocation.Allocator(self._initial_count)
 
+        # If there are any MultiTexCoord attributes, then a TexCoord attribute
+        # must be converted.
+        have_multi_texcoord = False
+        for attribute, _, _ in attribute_usages:
+            if isinstance(attribute, vertexattribute.MultiTexCoordAttribute):
+                have_multi_texcoord = True
+                break
+
         static_attributes = []
         attributes = []
         self.buffer_attributes = []   # list of (buffer, attributes)
         for attribute, usage, vbo in attribute_usages:
+            if (have_multi_texcoord and
+                isinstance(attribute, vertexattribute.TexCoordAttribute)):
+                attribute.convert_to_multi_tex_coord_attribute()
+
             if usage == GL_STATIC_DRAW:
                 # Group attributes for interleaved buffer
                 static_attributes.append(attribute)
@@ -170,7 +185,7 @@ class VertexDomain(object):
                 # Create non-interleaved buffer
                 attributes.append(attribute)
                 attribute.buffer = vertexbuffer.create_mappable_buffer(
-                    attribute.stride * self.allocator.capacity, 
+                    attribute.stride * self.allocator.capacity,
                     usage=usage, vbo=vbo)
                 attribute.buffer.element_size = attribute.stride
                 attribute.buffer.attributes = (attribute,)
@@ -187,6 +202,7 @@ class VertexDomain(object):
             self.buffer_attributes.append(
                 (buffer, static_attributes))
 
+            attributes.extend(static_attributes)
             for attribute in static_attributes:
                 attribute.buffer = buffer
 
@@ -196,11 +212,22 @@ class VertexDomain(object):
         for attribute in attributes:
             if isinstance(attribute, vertexattribute.GenericAttribute):
                 index = attribute.index
-                if 'generic' not in self.attributes:
-                    self.attributes['generic'] = {}
-                assert index not in self.attributes['generic'], \
+                # TODO create a name and use it (e.g. 'generic3')
+                # XXX this won't migrate; not documented.
+                if 'generic' not in self.attribute_names:
+                    self.attribute_names['generic'] = {}
+                assert index not in self.attribute_names['generic'], \
                     'More than one generic attribute with index %d' % index
                 self.attribute_names['generic'][index] = attribute
+            elif isinstance(attribute, vertexattribute.MultiTexCoordAttribute):
+                # XXX this won't migrate; not documented.
+                texture = attribute.texture
+                if 'multi_tex_coords' not in self.attribute_names:
+                    self.attribute_names['multi_tex_coords'] = []
+                assert texture not in self.attribute_names['multi_tex_coords'],\
+                    'More than one multi_tex_coord attribute for texture %d' % \
+                        texture
+                self.attribute_names['multi_tex_coords'].insert(texture,attribute)
             else:
                 name = attribute.plural
                 assert name not in self.attributes, \
@@ -211,13 +238,16 @@ class VertexDomain(object):
         # Break circular refs that Python GC seems to miss even when forced
         # collection.
         for attribute in self.attributes:
-            del attribute.buffer
+            try:
+                del attribute.buffer
+            except AttributeError:
+                pass
 
     def _safe_alloc(self, count):
         '''Allocate vertices, resizing the buffers if necessary.'''
         try:
             return self.allocator.alloc(count)
-        except allocation.AllocatorMemoryException, e:
+        except allocation.AllocatorMemoryException as e:
             capacity = _nearest_pow2(e.requested_capacity)
             self._version += 1
             for buffer, _ in self.buffer_attributes:
@@ -229,39 +259,39 @@ class VertexDomain(object):
         '''Reallocate vertices, resizing the buffers if necessary.'''
         try:
             return self.allocator.realloc(start, count, new_count)
-        except allocation.AllocatorMemoryException, e:
+        except allocation.AllocatorMemoryException as e:
             capacity = _nearest_pow2(e.requested_capacity)
             self._version += 1
             for buffer, _ in self.buffer_attributes:
                 buffer.resize(capacity * buffer.element_size)
             self.allocator.set_capacity(capacity)
-            return self.allocator.realloc(start, count, new_count) 
+            return self.allocator.realloc(start, count, new_count)
 
     def create(self, count):
-        '''Create a `VertexList` in this domain.
+        '''Create a :py:class:`VertexList` in this domain.
 
         :Parameters:
             `count` : int
                 Number of vertices to create.
 
-        :rtype: `VertexList`
+        :rtype: :py:class:`VertexList`
         '''
         start = self._safe_alloc(count)
         return VertexList(self, start, count)
 
     def draw(self, mode, vertex_list=None):
         '''Draw vertices in the domain.
-        
+
         If `vertex_list` is not specified, all vertices in the domain are
         drawn.  This is the most efficient way to render primitives.
 
-        If `vertex_list` specifies a `VertexList`, only primitives in that
-        list will be drawn.
+        If `vertex_list` specifies a :py:class:`VertexList`, only primitives in
+        that list will be drawn.
 
         :Parameters:
             `mode` : int
                 OpenGL drawing mode, e.g. ``GL_POINTS``, ``GL_LINES``, etc.
-            `vertex_list` : `VertexList`
+            `vertex_list` : `~pyglet.graphics.vertexdomain.VertexList`
                 Vertex list to draw, or ``None`` for all lists in this domain.
 
         '''
@@ -291,7 +321,7 @@ class VertexDomain(object):
             else:
                 for start, size in zip(starts, sizes):
                     glDrawArrays(mode, start, size)
-        
+
         for buffer, _ in self.buffer_attributes:
             buffer.unbind()
         glPopClientAttrib()
@@ -300,20 +330,20 @@ class VertexDomain(object):
         return not self.allocator.starts
 
     def __repr__(self):
-        return '<%s@%x %s>' % (self.__class__.__name__, id(self), 
+        return '<%s@%x %s>' % (self.__class__.__name__, id(self),
                                self.allocator)
 
 class VertexList(object):
-    '''A list of vertices within a `VertexDomain`.  Use
-    `VertexDomain.create` to construct this list.
+    '''A list of vertices within a :py:class:`VertexDomain`.  Use
+    :py:meth:`VertexDomain.create` to construct this list.
     '''
-    
+
     def __init__(self, domain, start, count):
         # TODO make private
         self.domain = domain
         self.start = start
         self.count = count
-    
+
     def get_size(self):
         '''Get the number of vertices in the list.
 
@@ -324,7 +354,7 @@ class VertexList(object):
     def get_domain(self):
         '''Get the domain this vertex list belongs to.
 
-        :rtype: `VertexDomain`
+        :rtype: :py:class:`VertexDomain`
         '''
         return self.domain
 
@@ -337,28 +367,34 @@ class VertexList(object):
 
         '''
         self.domain.draw(mode, self)
-    
+
     def resize(self, count):
         '''Resize this group.
-        
+
         :Parameters:
             `count` : int
-                New number of vertices in the list. 
+                New number of vertices in the list.
 
         '''
         new_start = self.domain._safe_realloc(self.start, self.count, count)
         if new_start != self.start:
             # Copy contents to new location
             for attribute in self.domain.attributes:
-                old = attribute.get_region(attribute.buffer, 
+                old = attribute.get_region(attribute.buffer,
                                            self.start, self.count)
-                new = attribute.get_region(attribute.buffer, 
+                new = attribute.get_region(attribute.buffer,
                                            new_start, self.count)
                 new.array[:] = old.array[:]
                 new.invalidate()
         self.start = new_start
         self.count = count
+
         self._colors_cache_version = None
+        self._fog_coords_cache_version = None
+        self._edge_flags_cache_version = None
+        self._normals_cache_version = None
+        self._secondary_colors_cache_version = None
+        self._tex_coords_cache_version = None
         self._vertices_cache_version = None
 
     def delete(self):
@@ -369,14 +405,14 @@ class VertexList(object):
         '''Move this group from its current domain and add to the specified
         one.  Attributes on domains must match.  (In practice, used to change
         parent state of some vertices).
-        
+
         :Parameters:
             `domain` : `VertexDomain`
                 Domain to migrate this vertex list to.
 
         '''
-        assert domain.attribute_names.keys() == \
-            self.domain.attribute_names.keys(), 'Domain attributes must match.'
+        assert list(domain.attribute_names.keys()) == \
+            list(self.domain.attribute_names.keys()), 'Domain attributes must match.'
 
         new_start = domain._safe_alloc(self.count)
         for key, old_attribute in self.domain.attribute_names.items():
@@ -391,7 +427,7 @@ class VertexList(object):
         self.domain.allocator.dealloc(self.start, self.count)
         self.domain = domain
         self.start = new_start
-        
+
         self._colors_cache_version = None
         self._fog_coords_cache_version = None
         self._edge_flags_cache_version = None
@@ -426,7 +462,7 @@ class VertexList(object):
 
     _colors_cache = None
     _colors_cache_version = None
-    colors = property(_get_colors, _set_colors, 
+    colors = property(_get_colors, _set_colors,
                       doc='''Array of color data.''')
 
     # ---
@@ -523,25 +559,62 @@ class VertexList(object):
     _tex_coords_cache_version = None
 
     def _get_tex_coords(self):
-        if (self._tex_coords_cache_version != self.domain._version):
-            domain = self.domain
-            attribute = domain.attribute_names['tex_coords']
-            self._tex_coords_cache = attribute.get_region(
-                attribute.buffer, self.start, self.count)
-            self._tex_coords_cache_version = domain._version
+        if 'multi_tex_coords' not in self.domain.attribute_names:
+            if (self._tex_coords_cache_version != self.domain._version):
+                domain = self.domain
+                attribute = domain.attribute_names['tex_coords']
+                self._tex_coords_cache = attribute.get_region(
+                    attribute.buffer, self.start, self.count)
+                self._tex_coords_cache_version = domain._version
 
-        region = self._tex_coords_cache
-        region.invalidate()
-        return region.array
+            region = self._tex_coords_cache
+            region.invalidate()
+            return region.array
+        else:
+            return None
 
     def _set_tex_coords(self, data):
-        self._get_tex_coords()[:] = data
+        if self._get_tex_coords() != None:
+            self._get_tex_coords()[:] = data
 
     tex_coords = property(_get_tex_coords, _set_tex_coords,
                           doc='''Array of texture coordinate data.''')
 
     # ---
-    
+
+    def _get_multi_tex_coords(self):
+        if 'tex_coords' not in self.domain.attribute_names:
+            if (self._tex_coords_cache_version != self.domain._version):
+                domain = self.domain
+                attribute = domain.attribute_names['multi_tex_coords']
+                self._tex_coords_cache = []
+                for a in attribute:
+                    self._tex_coords_cache.append(a.get_region(
+                        a.buffer, self.start, self.count))
+                self._tex_coords_cache_version = domain._version
+
+            region = self._tex_coords_cache
+            array = []
+            for a in region:
+                a.invalidate()
+                array.append(a.array)
+            return array
+        else:
+            return None
+
+    def _set_multi_tex_coords(self, data):
+        if self._get_multi_tex_coords() != None:
+            for a in range(0, len(self._tex_coords_cache),1):
+                if a > len(data):
+                    break
+                elif data[a] != None:
+                    self._tex_coords_cache[a].array[:] = data[a]
+
+    multi_tex_coords = property(_get_multi_tex_coords, _set_multi_tex_coords,
+                                doc='''Multi-array texture coordinate data.''')
+
+    # ---
+
     _vertices_cache = None
     _vertices_cache_version = None
 
@@ -559,7 +632,7 @@ class VertexList(object):
 
     def _set_vertices(self, data):
         self._get_vertices()[:] = data
-    
+
     vertices = property(_get_vertices, _set_vertices,
                         doc='''Array of vertex coordinate data.''')
 
@@ -575,19 +648,19 @@ class IndexedVertexDomain(VertexDomain):
         super(IndexedVertexDomain, self).__init__(attribute_usages)
 
         self.index_allocator = allocation.Allocator(self._initial_index_count)
-        
+
         self.index_gl_type = index_gl_type
         self.index_c_type = vertexattribute._c_types[index_gl_type]
         self.index_element_size = ctypes.sizeof(self.index_c_type)
         self.index_buffer = vertexbuffer.create_mappable_buffer(
-            self.index_allocator.capacity * self.index_element_size, 
+            self.index_allocator.capacity * self.index_element_size,
             target=GL_ELEMENT_ARRAY_BUFFER)
 
     def _safe_index_alloc(self, count):
         '''Allocate indices, resizing the buffers if necessary.'''
         try:
             return self.index_allocator.alloc(count)
-        except allocation.AllocatorMemoryException, e:
+        except allocation.AllocatorMemoryException as e:
             capacity = _nearest_pow2(e.requested_capacity)
             self._version += 1
             self.index_buffer.resize(capacity * self.index_element_size)
@@ -598,7 +671,7 @@ class IndexedVertexDomain(VertexDomain):
         '''Reallocate indices, resizing the buffers if necessary.'''
         try:
             return self.index_allocator.realloc(start, count, new_count)
-        except allocation.AllocatorMemoryException, e:
+        except allocation.AllocatorMemoryException as e:
             capacity = _nearest_pow2(e.requested_capacity)
             self._version += 1
             self.index_buffer.resize(capacity * self.index_element_size)
@@ -606,7 +679,7 @@ class IndexedVertexDomain(VertexDomain):
             return self.index_allocator.realloc(start, count, new_count)
 
     def create(self, count, index_count):
-        '''Create an `IndexedVertexList` in this domain.
+        '''Create an :py:class:`IndexedVertexList` in this domain.
 
         :Parameters:
             `count` : int
@@ -617,7 +690,7 @@ class IndexedVertexDomain(VertexDomain):
         '''
         start = self._safe_alloc(count)
         index_start = self._safe_index_alloc(index_count)
-        return IndexedVertexList(self, start, count, index_start, index_count) 
+        return IndexedVertexList(self, start, count, index_start, index_count)
 
     def get_index_region(self, start, count):
         '''Get a region of the index buffer.
@@ -637,12 +710,12 @@ class IndexedVertexDomain(VertexDomain):
 
     def draw(self, mode, vertex_list=None):
         '''Draw vertices in the domain.
-        
+
         If `vertex_list` is not specified, all vertices in the domain are
         drawn.  This is the most efficient way to render primitives.
 
-        If `vertex_list` specifies a `VertexList`, only primitives in that
-        list will be drawn.
+        If `vertex_list` specifies a :py:class:`VertexList`, only primitives in
+        that list will be drawn.
 
         :Parameters:
             `mode` : int
@@ -663,7 +736,7 @@ class IndexedVertexDomain(VertexDomain):
 
         if vertex_list is not None:
             glDrawElements(mode, vertex_list.index_count, self.index_gl_type,
-                self.index_buffer.ptr + 
+                self.index_buffer.ptr +
                     vertex_list.index_start * self.index_element_size)
         else:
             starts, sizes = self.index_allocator.get_allocated_regions()
@@ -675,27 +748,25 @@ class IndexedVertexDomain(VertexDomain):
                 glDrawElements(mode, sizes[0], self.index_gl_type,
                     self.index_buffer.ptr + starts[0])
             elif gl_info.have_version(1, 4):
-                if not isinstance(self.index_buffer, 
-                                  vertexbuffer.VertexBufferObject):
-                    starts = [s + self.index_buffer.ptr for s in starts]
-                starts = (GLuint * primcount)(*starts)
+                starts = [s * self.index_element_size + self.index_buffer.ptr for s in starts]
+                starts = ctypes.cast((GLuint * primcount)(*starts), ctypes.POINTER(ctypes.c_void_p))
                 sizes = (GLsizei * primcount)(*sizes)
-                glMultiDrawElements(mode, sizes, self.index_gl_type, starts,
+                glMultiDrawElements(mode, sizes, GL_UNSIGNED_INT, starts,
                                     primcount)
             else:
                 for start, size in zip(starts, sizes):
                     glDrawElements(mode, size, self.index_gl_type,
-                        self.index_buffer.ptr + 
+                        self.index_buffer.ptr +
                             start * self.index_element_size)
-        
+
         self.index_buffer.unbind()
         for buffer, _ in self.buffer_attributes:
             buffer.unbind()
         glPopClientAttrib()
 
 class IndexedVertexList(VertexList):
-    '''A list of vertices within an `IndexedVertexDomain` that are indexed.
-    Use `IndexedVertexDomain.create` to construct this list.
+    '''A list of vertices within an :py:class:`IndexedVertexDomain` that are
+    indexed. Use :py:meth:`IndexedVertexDomain.create` to construct this list.
     '''
     def __init__(self, domain, start, count, index_start, index_count):
         super(IndexedVertexList, self).__init__(domain, start, count)
@@ -711,9 +782,9 @@ class IndexedVertexList(VertexList):
 
         :Parameters:
             `count` : int
-                New number of vertices in the list. 
+                New number of vertices in the list.
             `index_count` : int
-                New number of indices in the list. 
+                New number of indices in the list.
 
         '''
         old_start = self.start
@@ -722,8 +793,8 @@ class IndexedVertexList(VertexList):
         # Change indices (because vertices moved)
         if old_start != self.start:
             diff = self.start - old_start
-            self.indices[:] = map(lambda i: i + diff, self.indices)
-        
+            self.indices[:] = [i + diff for i in self.indices]
+
         # Resize indices
         new_start = self.domain._safe_index_realloc(
             self.index_start, self.index_count, index_count)
@@ -742,6 +813,43 @@ class IndexedVertexList(VertexList):
         '''Delete this group.'''
         super(IndexedVertexList, self).delete()
         self.domain.index_allocator.dealloc(self.index_start, self.index_count)
+        
+    def migrate(self, domain):
+        '''Move this group from its current indexed domain and add to the 
+        specified one.  Attributes on domains must match.  (In practice, used 
+        to change parent state of some vertices).
+
+        :Parameters:
+            `domain` : `IndexedVertexDomain`
+                Indexed domain to migrate this vertex list to.
+
+        '''
+        old_start = self.start
+        old_domain = self.domain
+        super(IndexedVertexList, self).migrate(domain)
+
+        # Note: this code renumber the indices of the *original* domain
+        # because the vertices are in a new position in the new domain
+        if old_start != self.start:
+            diff = self.start - old_start
+            region = old_domain.get_index_region(self.index_start, 
+                                self.index_count)
+            old_indices = region.array
+            old_indices[:] = [i + diff for i in old_indices]
+            region.invalidate()
+                                                
+        # copy indices to new domain
+        old = old_domain.get_index_region(self.index_start, self.index_count)
+        # must delloc before calling safe_index_alloc or else problems when same
+        # batch is migrated to because index_start changes after dealloc
+        old_domain.index_allocator.dealloc(self.index_start, self.index_count)
+        new_start = self.domain._safe_index_alloc(self.index_count)
+        new = self.domain.get_index_region(new_start, self.index_count)
+        new.array[:] = old.array[:]
+        new.invalidate()
+        
+        self.index_start = new_start
+        self._indices_cache_version = None
 
     def _set_index_data(self, data):
         # TODO without region
@@ -768,5 +876,5 @@ class IndexedVertexList(VertexList):
 
     _indices_cache = None
     _indices_cache_version = None
-    indices = property(_get_indices, _set_indices, 
+    indices = property(_get_indices, _set_indices,
                        doc='''Array of index data.''')
