@@ -35,6 +35,10 @@
 
 These extend and correct ctypes functions.
 '''
+from __future__ import print_function
+from builtins import str
+from builtins import object
+from past.builtins import basestring
 
 __docformat__ = 'restructuredtext'
 __version__ = '$Id: $'
@@ -51,6 +55,14 @@ import pyglet
 _debug_lib = pyglet.options['debug_lib']
 _debug_trace = pyglet.options['debug_trace']
 
+_is_epydoc = hasattr(sys, 'is_epydoc') and sys.is_epydoc
+
+if pyglet.options['search_local_libs']:
+    script_path = pyglet.resource.get_script_home()
+    _local_lib_paths = [script_path, os.path.join(script_path, 'lib'),]
+else:
+    _local_lib_paths = None
+
 class _TraceFunction(object):
     def __init__(self, func):
         self.__dict__['_func'] = func
@@ -70,13 +82,26 @@ class _TraceFunction(object):
 class _TraceLibrary(object):
     def __init__(self, library):
         self._library = library
-        print library
+        print(library)
 
     def __getattr__(self, name):
         func = getattr(self._library, name)
         f = _TraceFunction(func)
         return f
 
+if _is_epydoc:
+    class LibraryMock(object):
+        """Mock library used when generating documentation."""
+        def __getattr__(self, name):
+            return LibraryMock()
+
+        def __setattr__(self, name, value):
+            pass
+
+        def __call__(self, *args, **kwargs):
+            return LibraryMock()
+
+
 class LibraryLoader(object):
     def load_library(self, *names, **kwargs):
         '''Find and load a library.  
@@ -86,34 +111,45 @@ class LibraryLoader(object):
 
         Raises ImportError if library is not found.
         '''
+        if _is_epydoc:
+            return LibraryMock()
+
         if 'framework' in kwargs and self.platform == 'darwin':
             return self.load_framework(kwargs['framework'])
+
+        if not names:
+            raise ImportError("No library name specified")
         
         platform_names = kwargs.get(self.platform, [])
-        if type(platform_names) in (str, unicode):
+        if isinstance(platform_names, basestring):
             platform_names = [platform_names]
         elif type(platform_names) is tuple:
             platform_names = list(platform_names)
 
-        if self.platform == 'linux2':
-            platform_names.extend(['lib%s.so' % n for n in names])
+        if self.platform.startswith('linux'):
+            for name in names:
+                libname = self.find_library(name)
+                platform_names.append(libname or 'lib%s.so' % name)
 
         platform_names.extend(names)
         for name in platform_names:
             try:
                 lib = ctypes.cdll.LoadLibrary(name)
                 if _debug_lib:
-                    print name
+                    print(name)
                 if _debug_trace:
                     lib = _TraceLibrary(lib)
                 return lib
-            except OSError:
+            except OSError as o:
+                if self.platform == "win32" and o.winerror != 126:
+                    print("Unexpected error loading library %s: %s" % (name, str(o)))
+                    raise
                 path = self.find_library(name)
                 if path:
                     try:
                         lib = ctypes.cdll.LoadLibrary(path)
                         if _debug_lib:
-                            print path
+                            print(path)
                         if _debug_trace:
                             lib = _TraceLibrary(lib)
                         return lib
@@ -123,7 +159,8 @@ class LibraryLoader(object):
 
     find_library = lambda self, name: ctypes.util.find_library(name)
 
-    platform = sys.platform
+    platform = pyglet.compat_platform
+    # this is only for library loading, don't include it in pyglet.platform
     if platform == 'cygwin':
         platform = 'win32'
 
@@ -137,6 +174,11 @@ class MachOLibraryLoader(LibraryLoader):
         else:
             self.ld_library_path = []
 
+        if _local_lib_paths:
+            # search first for local libs
+            self.ld_library_path = _local_lib_paths + self.ld_library_path
+            os.environ['LD_LIBRARY_PATH'] = ':'.join(self.ld_library_path)
+
         if 'DYLD_LIBRARY_PATH' in os.environ:
             self.dyld_library_path = os.environ['DYLD_LIBRARY_PATH'].split(':')
         else:
@@ -153,8 +195,8 @@ class MachOLibraryLoader(LibraryLoader):
  
     def find_library(self, path):
         '''Implements the dylib search as specified in Apple documentation:
-        
-        http://developer.apple.com/documentation/DeveloperTools/Conceptual/DynamicLibraries/Articles/DynamicLibraryUsageGuidelines.html
+
+        http://developer.apple.com/documentation/DeveloperTools/Conceptual/DynamicLibraries/100-Articles/DynamicLibraryUsageGuidelines.html
 
         Before commencing the standard search, the method first checks
         the bundle's ``Frameworks`` directory if the application is running
@@ -164,13 +206,24 @@ class MachOLibraryLoader(LibraryLoader):
         libname = os.path.basename(path)
         search_path = []
 
-        if hasattr(sys, 'frozen') and sys.frozen == 'macosx_app':
+        if '.' not in libname:
+            libname = 'lib' + libname + '.dylib'
+
+        # py2app support
+        if (hasattr(sys, 'frozen') and sys.frozen == 'macosx_app' and
+                'RESOURCEPATH' in os.environ):
             search_path.append(os.path.join(
                 os.environ['RESOURCEPATH'],
                 '..',
                 'Frameworks',
                 libname))
 
+        # pyinstaller.py sets sys.frozen to True, and puts dylibs in
+        # Contents/MacOS, which path pyinstaller puts in sys._MEIPASS
+        if (hasattr(sys, 'frozen') and hasattr(sys, '_MEIPASS') and
+                sys.frozen == True and pyglet.compat_platform == 'darwin'):
+            search_path.append(os.path.join(sys._MEIPASS, libname))
+
         if '/' in path:
             search_path.extend(
                 [os.path.join(p, libname) \
@@ -225,7 +278,7 @@ class MachOLibraryLoader(LibraryLoader):
         if realpath:
             lib = ctypes.cdll.LoadLibrary(realpath)
             if _debug_lib:
-                print realpath
+                print(realpath)
             if _debug_trace:
                 lib = _TraceLibrary(lib)
             return lib
@@ -234,6 +287,27 @@ class MachOLibraryLoader(LibraryLoader):
 
 class LinuxLibraryLoader(LibraryLoader):
     _ld_so_cache = None
+    _local_libs_cache = None
+
+    def _find_libs(self, directories):
+        cache = {}
+        lib_re = re.compile('lib(.*)\.so(?:$|\.)')
+        for dir in directories:
+            try:
+                for file in os.listdir(dir):
+                    match = lib_re.match(file)
+                    if match:
+                        # Index by filename
+                        path = os.path.join(dir, file)
+                        if file not in cache:
+                            cache[file] = path
+                        # Index by library name
+                        library = match.group(1)
+                        if library not in cache:
+                            cache[library] = path
+            except OSError:
+                pass
+        return cache
 
     def _create_ld_so_cache(self):
         # Recreate search path followed by ld.so.  This is going to be
@@ -250,37 +324,24 @@ class LinuxLibraryLoader(LibraryLoader):
             pass
 
         try:
-            directories.extend([dir.strip() for dir in open('/etc/ld.so.conf')])
+            with open('/etc/ld.so.conf') as fid:
+                directories.extend([dir.strip() for dir in fid])
         except IOError:
             pass
 
         directories.extend(['/lib', '/usr/lib'])
 
-        cache = {}
-        lib_re = re.compile('lib(.*)\.so')
-        for dir in directories:
-            try:
-                for file in os.listdir(dir):
-                    if '.so' not in file:
-                        continue
-
-                    # Index by filename
-                    path = os.path.join(dir, file)
-                    if file not in cache:
-                        cache[file] = path
-
-                    # Index by library name
-                    match = lib_re.match(file)
-                    if match:
-                        library = match.group(1)
-                        if library not in cache:
-                            cache[library] = path
-            except OSError:
-                pass
-
-        self._ld_so_cache = cache
+        self._ld_so_cache = self._find_libs(directories)
 
     def find_library(self, path):
+
+        # search first for local libs
+        if _local_lib_paths:
+            if not self._local_libs_cache:
+                self._local_libs_cache = self._find_libs(_local_lib_paths)
+            if path in self._local_libs_cache:
+                return self._local_libs_cache[path]
+
         # ctypes tries ldconfig, gcc and objdump.  If none of these are
         # present, we implement the ld-linux.so search path as described in
         # the man page.
@@ -294,9 +355,9 @@ class LinuxLibraryLoader(LibraryLoader):
 
         return self._ld_so_cache.get(path)
 
-if sys.platform == 'darwin':
+if pyglet.compat_platform == 'darwin':
     loader = MachOLibraryLoader()
-elif sys.platform == 'linux2':
+elif pyglet.compat_platform.startswith('linux'):
     loader = LinuxLibraryLoader()
 else:
     loader = LibraryLoader()
