@@ -2,14 +2,14 @@
 # pyglet
 # Copyright (c) 2006-2008 Alex Holkner
 # All rights reserved.
-# 
+#
 # Redistribution and use in source and binary forms, with or without
-# modification, are permitted provided that the following conditions 
+# modification, are permitted provided that the following conditions
 # are met:
 #
 #  * Redistributions of source code must retain the above copyright
 #    notice, this list of conditions and the following disclaimer.
-#  * Redistributions in binary form must reproduce the above copyright 
+#  * Redistributions in binary form must reproduce the above copyright
 #    notice, this list of conditions and the following disclaimer in
 #    the documentation and/or other materials provided with the
 #    distribution.
@@ -32,30 +32,30 @@
 # POSSIBILITY OF SUCH DAMAGE.
 # ----------------------------------------------------------------------------
 
-'''Load application resources from a known path.
+"""Load application resources from a known path.
 
 Loading resources by specifying relative paths to filenames is often
 problematic in Python, as the working directory is not necessarily the same
 directory as the application's script files.
 
 This module allows applications to specify a search path for resources.
-Relative paths are taken to be relative to the application's __main__ module.
-ZIP files can appear on the path; they will be searched inside.  The resource
-module also behaves as expected when applications are bundled using py2exe or
-py2app.
+Relative paths are taken to be relative to the application's ``__main__``
+module. ZIP files can appear on the path; they will be searched inside.  The
+resource module also behaves as expected when applications are bundled using py2exe or py2app.
 
-As well as providing file references (with the `file` function), the resource
-module also contains convenience functions for loading images, textures,
-fonts, media and documents.
+As well as providing file references (with the :py:func:`file` function), the
+resource module also contains convenience functions for loading images,
+textures, fonts, media and documents.
 
 3rd party modules or packages not bound to a specific application should
-construct their own `Loader` instance and override the path to use the
+construct their own :py:class:`Loader` instance and override the path to use the
 resources in the module's directory.
 
 Path format
 ^^^^^^^^^^^
 
-The resource path `path` (see also `Loader.__init__` and `Loader.path`)
+The resource path :py:attr:`path` (see also :py:meth:`Loader.__init__` and
+:py:meth:`Loader.path`)
 is a list of locations to search for resources.  Locations are searched in the
 order given in the path.  If a location is not valid (for example, if the
 directory does not exist), it is skipped.
@@ -67,42 +67,51 @@ script home.  Some examples::
 
     # Search just the `res` directory, assumed to be located alongside the
     # main script file.
-    path = ['res'] 
+    path = ['res']
 
     # Search the directory containing the module `levels.level1`, followed
-    # by the `res` directory.
-    path = ['@levels.level1', 'res']
+    # by the `res/images` directory.
+    path = ['@levels.level1', 'res/images']
 
-Paths are always case-sensitive, even if the filesystem is not.  This
-avoids a common programmer error when porting applications between platforms.
+Paths are always case-sensitive and forward slashes are always used as path
+separators, even in cases when the filesystem or platform does not do this.
+This avoids a common programmer error when porting applications between
+platforms.
 
 The default path is ``['.']``.  If you modify the path, you must call
-`reindex`.
+:py:func:`reindex`.
 
-:since: pyglet 1.1
-'''
+.. versionadded:: 1.1
+"""
+from future import standard_library
+
+standard_library.install_aliases()
+from builtins import object
+from past.builtins import basestring
 
 __docformat__ = 'restructuredtext'
 __version__ = '$Id: $'
 
-import operator
 import os
 import weakref
 import sys
 import zipfile
-import StringIO
 
 import pyglet
+from pyglet.compat import BytesIO
+
 
 class ResourceNotFoundException(Exception):
-    '''The named resource was not found on the search path.'''
+    """The named resource was not found on the search path."""
+
     def __init__(self, name):
         message = ('Resource "%s" was not found on the path.  '
-            'Ensure that the filename has the correct captialisation.') % name
+                   'Ensure that the filename has the correct captialisation.') % name
         Exception.__init__(self, message)
 
+
 def get_script_home():
-    '''Get the directory containing the program entry module.
+    """Get the directory containing the program entry module.
 
     For ordinary Python scripts, this is the directory containing the
     ``__main__`` module.  For executables created with py2exe the result is
@@ -113,32 +122,46 @@ def get_script_home():
     If none of the above cases apply and the file for ``__main__`` cannot
     be determined the working directory is returned.
 
-    :rtype: str
-    '''
+    When the script is being run by a Python profiler, this function
+    may return the directory where the profiler is running instead of
+    the directory of the real script. To workaround this behaviour the
+    full path to the real script can be specified in :py:attr:`pyglet.resource.path`.
 
+    :rtype: str
+    """
     frozen = getattr(sys, 'frozen', None)
-    if frozen in ('windows_exe', 'console_exe'):
+    meipass = getattr(sys, '_MEIPASS', None)
+    if meipass:
+        # PyInstaller
+        return meipass
+    elif frozen in ('windows_exe', 'console_exe'):
         return os.path.dirname(sys.executable)
     elif frozen == 'macosx_app':
+        # py2app
         return os.environ['RESOURCEPATH']
     else:
         main = sys.modules['__main__']
         if hasattr(main, '__file__'):
-            return os.path.dirname(main.__file__)
+            return os.path.dirname(os.path.abspath(main.__file__))
+        else:
+            if 'python' in os.path.basename(sys.executable):
+                # interactive
+                return os.getcwd()
+            else:
+                # cx_Freeze
+                return os.path.dirname(sys.executable)
 
-    # Probably interactive
-    return ''
 
 def get_settings_path(name):
-    '''Get a directory to save user preferences.
+    """Get a directory to save user preferences.
 
     Different platforms have different conventions for where to save user
     preferences, saved games, and settings.  This function implements those
     conventions.  Note that the returned path may not exist: applications
     should use ``os.makedirs`` to construct it if desired.
 
-    On Linux, a hidden directory `name` in the user's home directory is
-    returned.
+    On Linux, a directory `name` in the user's configuration directory is
+    returned (usually under ``~/.config``).
 
     On Windows (including under Cygwin) the `name` directory in the user's
     ``Application Settings`` directory is returned.
@@ -151,26 +174,34 @@ def get_settings_path(name):
             The name of the application.
 
     :rtype: str
-    '''
-    if sys.platform in ('cygwin', 'win32'):
+    """
+
+    if pyglet.compat_platform in ('cygwin', 'win32'):
         if 'APPDATA' in os.environ:
             return os.path.join(os.environ['APPDATA'], name)
         else:
             return os.path.expanduser('~/%s' % name)
-    elif sys.platform == 'darwin':
+    elif pyglet.compat_platform == 'darwin':
         return os.path.expanduser('~/Library/Application Support/%s' % name)
+    elif pyglet.compat_platform.startswith('linux'):
+        if 'XDG_CONFIG_HOME' in os.environ:
+            return os.path.join(os.environ['XDG_CONFIG_HOME'], name)
+        else:
+            return os.path.expanduser('~/.config/%s' % name)
     else:
         return os.path.expanduser('~/.%s' % name)
 
+
 class Location(object):
-    '''Abstract resource location.
+    """Abstract resource location.
 
     Given a location, a file can be loaded from that location with the `open`
     method.  This provides a convenient way to specify a path to load files
     from, and not necessarily have that path reside on the filesystem.
-    '''
+    """
+
     def open(self, filename, mode='rb'):
-        '''Open a file at this locaiton.
+        """Open a file at this location.
 
         :Parameters:
             `filename` : str
@@ -182,29 +213,33 @@ class Location(object):
                 filesystem make use of this parameter; others ignore it.
 
         :rtype: file object
-        '''
+        """
         raise NotImplementedError('abstract')
 
+
 class FileLocation(Location):
-    '''Location on the filesystem.
-    '''
+    """Location on the filesystem.
+    """
+
     def __init__(self, path):
-        '''Create a location given a relative or absolute path.
+        """Create a location given a relative or absolute path.
 
         :Parameters:
             `path` : str
                 Path on the filesystem.
-        '''
+        """
         self.path = path
 
     def open(self, filename, mode='rb'):
         return open(os.path.join(self.path, filename), mode)
 
+
 class ZIPLocation(Location):
-    '''Location within a ZIP file.
-    '''
+    """Location within a ZIP file.
+    """
+
     def __init__(self, zip, dir):
-        '''Create a location given an open ZIP file and a path within that
+        """Create a location given an open ZIP file and a path within that
         file.
 
         :Parameters:
@@ -214,39 +249,44 @@ class ZIPLocation(Location):
                 A path within that ZIP file.  Can be empty to specify files at
                 the top level of the ZIP file.
 
-        '''
+        """
         self.zip = zip
         self.dir = dir
-        
+
     def open(self, filename, mode='rb'):
-        path = os.path.join(self.dir, filename)
+        if self.dir:
+            path = self.dir + '/' + filename
+        else:
+            path = filename
         text = self.zip.read(path)
-        return StringIO.StringIO(text)
-        
+        return BytesIO(text)
+
+
 class URLLocation(Location):
-    '''Location on the network.
+    """Location on the network.
 
     This class uses the ``urlparse`` and ``urllib2`` modules to open files on
     the network given a URL.
-    '''
+    """
+
     def __init__(self, base_url):
-        '''Create a location given a base URL.
+        """Create a location given a base URL.
 
         :Parameters:
             `base_url` : str
                 URL string to prepend to filenames.
 
-        '''
+        """
         self.base = base_url
 
     def open(self, filename, mode='rb'):
-        import urlparse
-        import urllib2
-        url = urlparse.urljoin(self.base, filename)
-        return urllib2.urlopen(url)
+        import urllib.request, urllib.error, urllib.parse
+        url = urllib.parse.urljoin(self.base, filename)
+        return urllib.request.urlopen(url)
+
 
 class Loader(object):
-    '''Load program resource files from disk.
+    """Load program resource files from disk.
 
     The loader contains a search path which can include filesystem
     directories, ZIP archives and Python packages.
@@ -259,9 +299,9 @@ class Loader(object):
             Base resource location, defaulting to the location of the
             application script.
 
-    '''
+    """
     def __init__(self, path=None, script_home=None):
-        '''Create a loader for the given path.
+        """Create a loader for the given path.
 
         If no path is specified it defaults to ``['.']``; that is, just the
         program directory.
@@ -275,31 +315,35 @@ class Loader(object):
                 Base location of relative files.  Defaults to the result of
                 `get_script_home`.
 
-        '''
+        """
         if path is None:
             path = ['.']
-        if type(path) in (str, unicode):
+        if isinstance(path, basestring):
             path = [path]
         self.path = list(path)
         if script_home is None:
             script_home = get_script_home()
         self._script_home = script_home
-        self.reindex()
-
-        # Map name to image
-        self._cached_textures = weakref.WeakValueDictionary()
-        self._cached_images = weakref.WeakValueDictionary()
-        self._cached_animations = weakref.WeakValueDictionary()
+        self._index = None
 
         # Map bin size to list of atlases
         self._texture_atlas_bins = {}
 
+    def _require_index(self):
+        if self._index is None:
+            self.reindex()
+
     def reindex(self):
-        '''Refresh the file index.
+        """Refresh the file index.
 
         You must call this method if `path` is changed or the filesystem
         layout changes.
-        '''
+        """
+        # map name to image etc.
+        self._cached_textures = weakref.WeakValueDictionary()
+        self._cached_images = weakref.WeakValueDictionary()
+        self._cached_animations = weakref.WeakValueDictionary()
+
         self._index = {}
         for path in self.path:
             if path.startswith('@'):
@@ -317,7 +361,7 @@ class Loader(object):
                 if hasattr(module, '__file__'):
                     path = os.path.dirname(module.__file__)
                 else:
-                    path = '' # interactive
+                    path = ''  # interactive
             elif not os.path.isabs(path):
                 # Add script base unless absolute
                 assert '\\' not in path, \
@@ -326,13 +370,17 @@ class Loader(object):
 
             if os.path.isdir(path):
                 # Filesystem directory
-                path = path.rstrip(os.path.pathsep)
+                path = path.rstrip(os.path.sep)
                 location = FileLocation(path)
                 for dirpath, dirnames, filenames in os.walk(path):
                     dirpath = dirpath[len(path) + 1:]
                     # Force forward slashes for index
                     if dirpath:
-                        dirpath = '/'.join(os.path.split(dirpath))
+                        parts = [part
+                                 for part
+                                 in dirpath.split(os.sep)
+                                 if part is not None]
+                        dirpath = '/'.join(parts)
                     for filename in filenames:
                         if dirpath:
                             index_name = dirpath + '/' + filename
@@ -342,9 +390,15 @@ class Loader(object):
             else:
                 # Find path component that is the ZIP file.
                 dir = ''
+                old_path = None
                 while path and not os.path.isfile(path):
+                    old_path = path
                     path, tail_dir = os.path.split(path)
+                    if path == old_path:
+                        break
                     dir = '/'.join((tail_dir, dir))
+                if path == old_path:
+                    continue
                 dir = dir.rstrip('/')
 
                 # path is a ZIP file, dir resides within ZIP
@@ -352,12 +406,12 @@ class Loader(object):
                     zip = zipfile.ZipFile(path, 'r')
                     location = ZIPLocation(zip, dir)
                     for zip_name in zip.namelist():
-                        #zip_name_dir, zip_name = os.path.split(zip_name)
-                        #assert '\\' not in name_dir
-                        #assert not name_dir.endswith('/')
+                        # zip_name_dir, zip_name = os.path.split(zip_name)
+                        # assert '\\' not in name_dir
+                        # assert not name_dir.endswith('/')
                         if zip_name.startswith(dir):
                             if dir:
-                                zip_name = zip_name[len(dir)+1:]
+                                zip_name = zip_name[len(dir) + 1:]
                             self._index_file(zip_name, location)
 
     def _index_file(self, name, location):
@@ -365,7 +419,7 @@ class Loader(object):
             self._index[name] = location
 
     def file(self, name, mode='rb'):
-        '''Load a resource.
+        """Load a resource.
 
         :Parameters:
             `name` : str
@@ -375,7 +429,8 @@ class Loader(object):
                 with the meaning as for the builtin ``open`` function.
 
         :rtype: file object
-        '''
+        """
+        self._require_index()
         try:
             location = self._index[name]
             return location.open(name, mode)
@@ -383,7 +438,7 @@ class Loader(object):
             raise ResourceNotFoundException(name)
 
     def location(self, name):
-        '''Get the location of a resource.
+        """Get the location of a resource.
 
         This method is useful for opening files referenced from a resource.
         For example, an HTML file loaded as a resource might reference some
@@ -395,14 +450,15 @@ class Loader(object):
                 Filename of the resource to locate.
 
         :rtype: `Location`
-        '''
+        """
+        self._require_index()
         try:
             return self._index[name]
         except KeyError:
             raise ResourceNotFoundException(name)
 
     def add_font(self, name):
-        '''Add a font resource to the application.
+        """Add a font resource to the application.
 
         Fonts not installed on the system must be added to pyglet before they
         can be used with `font.load`.  Although the font is added with
@@ -416,14 +472,23 @@ class Loader(object):
             `name` : str
                 Filename of the font resource to add.
 
-        '''
+        """
+        self._require_index()
         from pyglet import font
         file = self.file(name)
         font.add_file(file)
 
-    def _alloc_image(self, name):
+    def _alloc_image(self, name, atlas=True):
         file = self.file(name)
-        img = pyglet.image.load(name, file=file)
+        try:
+            img = pyglet.image.load(name, file=file)
+        finally:
+            file.close()
+
+        if not atlas:
+            return img.get_texture(True)
+
+        # find an atlas suitable for the image
         bin = self._get_texture_atlas_bin(img.width, img.height)
         if bin is None:
             return img.get_texture(True)
@@ -431,33 +496,35 @@ class Loader(object):
         return bin.add(img)
 
     def _get_texture_atlas_bin(self, width, height):
-        '''A heuristic for determining the atlas bin to use for a given image
+        """A heuristic for determining the atlas bin to use for a given image
         size.  Returns None if the image should not be placed in an atlas (too
         big), otherwise the bin (a list of TextureAtlas).
-        ''' 
+        """
         # Large images are not placed in an atlas
-        if width > 128 or height > 128:
+        max_texture_size = pyglet.image.atlas.get_max_texture_size()
+        max_size = min(1024, max_texture_size / 2)
+        if width > max_size or height > max_size:
             return None
 
-        # Group images with small height separately to larger height (as the
-        # allocator can't stack within a single row).
+        # Group images with small height separately to larger height
+        # (as the allocator can't stack within a single row).
         bin_size = 1
-        if height > 32:
+        if height > max_size / 4:
             bin_size = 2
 
         try:
-            bin = self._texture_atlas_bins[bin_size]
+            texture_bin = self._texture_atlas_bins[bin_size]
         except KeyError:
-            bin = self._texture_atlas_bins[bin_size] = \
+            texture_bin = self._texture_atlas_bins[bin_size] =\
                 pyglet.image.atlas.TextureBin()
 
-        return bin
+        return texture_bin
 
-    def image(self, name, flip_x=False, flip_y=False, rotate=0):
-        '''Load an image with optional transformation.
+    def image(self, name, flip_x=False, flip_y=False, rotate=0, atlas=True):
+        """Load an image with optional transformation.
 
         This is similar to `texture`, except the resulting image will be
-        packed into a `TextureBin` if it is an appropriate size for packing.
+        packed into a :py:class:`~pyglet.image.atlas.TextureBin` if it is an appropriate size for packing.
         This is more efficient than loading images into separate textures.
 
         :Parameters:
@@ -469,24 +536,31 @@ class Loader(object):
                 If True, the returned image will be flipped vertically.
             `rotate` : int
                 The returned image will be rotated clockwise by the given
-                number of degrees (a mulitple of 90).
+                number of degrees (a multiple of 90).
+            `atlas` : bool
+                If True, the image will be loaded into an atlas managed by
+                pyglet. If atlas loading is not appropriate for specific
+                texturing reasons (e.g. border control is required) then set
+                this argument to False.
 
         :rtype: `Texture`
-        :return: A complete texture if the image is large, otherwise a
-            `TextureRegion` of a texture atlas.
-        '''
+        :return: A complete texture if the image is large or not in an atlas,
+            otherwise a :py:class:`~pyglet.image.TextureRegion` of a texture atlas.
+        """
+        self._require_index()
         if name in self._cached_images:
             identity = self._cached_images[name]
         else:
-            identity = self._cached_images[name] = self._alloc_image(name)
+            identity = self._cached_images[name] = self._alloc_image(name,
+                                                                     atlas=atlas)
 
         if not rotate and not flip_x and not flip_y:
             return identity
-                
+
         return identity.get_transform(flip_x, flip_y, rotate)
 
     def animation(self, name, flip_x=False, flip_y=False, rotate=0):
-        '''Load an animation with optional transformation.
+        """Load an animation with optional transformation.
 
         Animations loaded from the same source but with different
         transformations will use the same textures.
@@ -500,15 +574,16 @@ class Loader(object):
                 If True, the returned image will be flipped vertically.
             `rotate` : int
                 The returned image will be rotated clockwise by the given
-                number of degrees (a mulitple of 90).
+                number of degrees (a multiple of 90).
 
-        :rtype: `Animation`
-        '''
+        :rtype: :py:class:`~pyglet.image.Animation`
+        """
+        self._require_index()
         try:
             identity = self._cached_animations[name]
         except KeyError:
             animation = pyglet.image.load_animation(name, self.file(name))
-            bin = self._get_texture_atlas_bin(animation.get_max_width(), 
+            bin = self._get_texture_atlas_bin(animation.get_max_width(),
                                               animation.get_max_height())
             if bin:
                 animation.add_to_texture_bin(bin)
@@ -517,41 +592,44 @@ class Loader(object):
 
         if not rotate and not flip_x and not flip_y:
             return identity
-                
+
         return identity.get_transform(flip_x, flip_y, rotate)
-       
+
     def get_cached_image_names(self):
-        '''Get a list of image filenames that have been cached.
+        """Get a list of image filenames that have been cached.
 
         This is useful for debugging and profiling only.
 
         :rtype: list
         :return: List of str
-        '''
-        return self._cached_images.keys()
+        """
+        self._require_index()
+        return list(self._cached_images.keys())
 
     def get_cached_animation_names(self):
-        '''Get a list of animation filenames that have been cached.
+        """Get a list of animation filenames that have been cached.
 
         This is useful for debugging and profiling only.
 
         :rtype: list
         :return: List of str
-        '''
-        return self._cached_animations.keys()
+        """
+        self._require_index()
+        return list(self._cached_animations.keys())
 
     def get_texture_bins(self):
-        '''Get a list of texture bins in use.
+        """Get a list of texture bins in use.
 
         This is useful for debugging and profiling only.
 
         :rtype: list
-        :return: List of `TextureBin`
-        '''
-        return self._texture_atlas_bins.values()
-       
+        :return: List of :py:class:`~pyglet.image.atlas.TextureBin`
+        """
+        self._require_index()
+        return list(self._texture_atlas_bins.values())
+
     def media(self, name, streaming=True):
-        '''Load a sound or video resource.
+        """Load a sound or video resource.
 
         The meaning of `streaming` is as for `media.load`.  Compressed
         sources cannot be streamed (that is, video and compressed audio
@@ -565,7 +643,8 @@ class Loader(object):
                 it should be entirely decoded into memory immediately.
 
         :rtype: `media.Source`
-        '''
+        """
+        self._require_index()
         from pyglet import media
         try:
             location = self._index[name]
@@ -581,10 +660,10 @@ class Loader(object):
             raise ResourceNotFoundException(name)
 
     def texture(self, name):
-        '''Load a texture.
+        """Load a texture.
 
         The named image will be loaded as a single OpenGL texture.  If the
-        dimensions of the image are not powers of 2 a `TextureRegion` will
+        dimensions of the image are not powers of 2 a :py:class:`~pyglet.image.TextureRegion` will
         be returned.
 
         :Parameters:
@@ -592,7 +671,8 @@ class Loader(object):
                 Filename of the image resource to load.
 
         :rtype: `Texture`
-        '''
+        """
+        self._require_index()
         if name in self._cached_textures:
             return self._cached_textures[name]
 
@@ -602,19 +682,20 @@ class Loader(object):
         return texture
 
     def html(self, name):
-        '''Load an HTML document.
+        """Load an HTML document.
 
         :Parameters:
             `name` : str
                 Filename of the HTML resource to load.
 
         :rtype: `FormattedDocument`
-        '''
+        """
+        self._require_index()
         file = self.file(name)
-        return pyglet.text.decode_html(file.read(), self.location(name))
+        return pyglet.text.load(name, file, 'text/html')
 
     def attributed(self, name):
-        '''Load an attributed text document.
+        """Load an attributed text document.
 
         See `pyglet.text.formats.attributed` for details on this format.
 
@@ -623,28 +704,32 @@ class Loader(object):
                 Filename of the attribute text resource to load.
 
         :rtype: `FormattedDocument`
-        '''
+        """
+        self._require_index()
         file = self.file(name)
         return pyglet.text.load(name, file, 'text/vnd.pyglet-attributed')
 
     def text(self, name):
-        '''Load a plain text document.
+        """Load a plain text document.
 
         :Parameters:
             `name` : str
                 Filename of the plain text resource to load.
 
         :rtype: `UnformattedDocument`
-        '''
+        """
+        self._require_index()
         file = self.file(name)
         return pyglet.text.load(name, file, 'text/plain')
 
     def get_cached_texture_names(self):
-        '''Get the names of textures currently cached.
+        """Get the names of textures currently cached.
 
         :rtype: list of str
-        '''
-        return self._cached_textures.keys()
+        """
+        self._require_index()
+        return list(self._cached_textures.keys())
+
 
 #: Default resource search path.
 #:
@@ -656,15 +741,18 @@ class Loader(object):
 #: :type: list of str
 path = []
 
+
 class _DefaultLoader(Loader):
-    def _get_path(self):
+
+    @property
+    def path(self):
         return path
 
-    def _set_path(self, value):
+    @path.setter
+    def path(self, value):
         global path
         path = value
 
-    path = property(_get_path, _set_path)
 
 _default_loader = _DefaultLoader()
 reindex = _default_loader.reindex
