@@ -2,14 +2,14 @@
 # pyglet
 # Copyright (c) 2006-2008 Alex Holkner
 # All rights reserved.
-# 
+#
 # Redistribution and use in source and binary forms, with or without
-# modification, are permitted provided that the following conditions 
+# modification, are permitted provided that the following conditions
 # are met:
 #
 #  * Redistributions of source code must retain the above copyright
 #    notice, this list of conditions and the following disclaimer.
-#  * Redistributions in binary form must reproduce the above copyright 
+#  * Redistributions in binary form must reproduce the above copyright
 #    notice, this list of conditions and the following disclaimer in
 #    the documentation and/or other materials provided with the
 #    distribution.
@@ -32,7 +32,7 @@
 # POSSIBILITY OF SUCH DAMAGE.
 # ----------------------------------------------------------------------------
 
-'''Key constants and utilities for pyglet.window.
+"""Key constants and utilities for pyglet.window.
 
 Usage::
 
@@ -58,13 +58,17 @@ Usage::
         # Modifiers:
         if modifiers & key.MOD_CTRL:
 
-'''
+"""
+from builtins import str
+
+from pyglet import compat_platform
 
 __docformat__ = 'restructuredtext'
-__version__ = '$Id: key.py 1919 2008-03-19 07:47:01Z Alex.Holkner $'
+__version__ = '$Id$'
+
 
 class KeyStateHandler(dict):
-    '''Simple handler that tracks the state of keys on the keyboard. If a
+    """Simple handler that tracks the state of keys on the keyboard. If a
     key is pressed then this handler holds a True value for it.
 
     For example::
@@ -80,16 +84,19 @@ class KeyStateHandler(dict):
         >>> keyboard[key.DOWN]
         False
 
-    '''
+    """
     def on_key_press(self, symbol, modifiers):
         self[symbol] = True
+
     def on_key_release(self, symbol, modifiers):
         self[symbol] = False
+
     def __getitem__(self, key):
         return self.get(key, False)
 
+
 def modifiers_string(modifiers):
-    '''Return a string describing a set of modifiers.
+    """Return a string describing a set of modifiers.
 
     Example::
 
@@ -101,7 +108,7 @@ def modifiers_string(modifiers):
             Bitwise combination of modifier constants.
 
     :rtype: str
-    '''
+    """
     mod_names = []
     if modifiers & MOD_SHIFT:
         mod_names.append('MOD_SHIFT')
@@ -119,13 +126,16 @@ def modifiers_string(modifiers):
         mod_names.append('MOD_COMMAND')
     if modifiers & MOD_OPTION:
         mod_names.append('MOD_OPTION')
+    if modifiers & MOD_FUNCTION:
+        mod_names.append('MOD_FUNCTION')
     return '|'.join(mod_names)
 
+
 def symbol_string(symbol):
-    '''Return a string describing a key symbol.
+    """Return a string describing a key symbol.
 
     Example::
-        
+
         >>> symbol_string(BACKSPACE)
         'BACKSPACE'
 
@@ -134,18 +144,19 @@ def symbol_string(symbol):
             Symbolic key constant.
 
     :rtype: str
-    '''
+    """
     if symbol < 1 << 32:
         return _key_names.get(symbol, str(symbol))
     else:
         return 'user_key(%x)' % (symbol >> 32)
 
+
 def motion_string(motion):
-    '''Return a string describing a text motion.
+    """Return a string describing a text motion.
 
     Example::
-        
-        >>> motion_string(MOTION_NEXT_WORD):
+
+        >>> motion_string(MOTION_NEXT_WORD)
         'MOTION_NEXT_WORD'
 
     :Parameters:
@@ -153,19 +164,20 @@ def motion_string(motion):
             Text motion constant.
 
     :rtype: str
-    '''
+    """
     return _motion_names.get(motion, str(motion))
 
+
 def user_key(scancode):
-    '''Return a key symbol for a key not supported by pyglet.  
-    
+    """Return a key symbol for a key not supported by pyglet.
+
     This can be used to map virtual keys or scancodes from unsupported
     keyboard layouts into a machine-specific symbol.  The symbol will
     be meaningless on any other machine, or under a different keyboard layout.
 
     Applications should use user-keys only when user explicitly binds them
     (for example, mapping keys to actions in a game options screen).
-    '''
+    """
     assert scancode > 0
     return scancode << 32
 
@@ -179,13 +191,13 @@ MOD_WINDOWS     = 1 << 5
 MOD_COMMAND     = 1 << 6
 MOD_OPTION      = 1 << 7
 MOD_SCROLLLOCK  = 1 << 8
+MOD_FUNCTION    = 1 << 9
 
 #: Accelerator modifier.  On Windows and Linux, this is ``MOD_CTRL``, on
 #: Mac OS X it's ``MOD_COMMAND``.
-MOD_ACCEL       = MOD_CTRL
-import sys as _sys
-if _sys.platform == 'darwin':
-    MOD_ACCEL   = MOD_COMMAND
+MOD_ACCEL = MOD_CTRL
+if compat_platform == 'darwin':
+    MOD_ACCEL = MOD_COMMAND
 
 
 # Key symbol constants
@@ -229,6 +241,7 @@ HELP          = 0xff6a
 BREAK         = 0xff6b
 MODESWITCH    = 0xff7e
 SCRIPTSWITCH  = 0xff7e
+FUNCTION      = 0xffd2
 
 # Text motion constants: these are allowed to clash with key constants
 MOTION_UP                = UP
@@ -304,6 +317,10 @@ F13           = 0xffca
 F14           = 0xffcb
 F15           = 0xffcc
 F16           = 0xffcd
+F17           = 0xffce
+F18           = 0xffcf
+F19           = 0xffd0
+F20           = 0xffd1
 
 # Modifiers
 LSHIFT        = 0xffe1
@@ -319,8 +336,8 @@ LWINDOWS      = 0xffeb
 RWINDOWS      = 0xffec
 LCOMMAND      = 0xffed
 RCOMMAND      = 0xffee
-LOPTION       = 0xffd0
-ROPTION       = 0xffd1
+LOPTION       = 0xffef
+ROPTION       = 0xfff0
 
 # Latin-1
 SPACE         = 0x020
@@ -397,7 +414,7 @@ ASCIITILDE    = 0x07e
 
 _key_names = {}
 _motion_names = {}
-for _name, _value in locals().items():
+for _name, _value in locals().copy().items():
     if _name[:2] != '__' and _name.upper() == _name and \
        not _name.startswith('MOD_'):
         if _name.startswith('MOTION_'):
